import { Role } from './ad.types';
import { Target } from './candidate.types';
import { Actor } from './value-objects/actor.value-object';
import { Point } from './value-objects/point.value-object';
import { WayStep } from './value-objects/waystep.value-object';

export class CarpoolPathCreator {
  constructor(
    private readonly driverWaypoints: Point[],
    private readonly passengerWaypoints: Point[],
  ) {}

  public createCarpoolPath = (): WayStep[] =>
    this._createMixedWaysteps(
      this._createDriverWaysteps(),
      this._createPassengerWaysteps(),
    );

  private _createDriverWaysteps = (): WayStep[] =>
    this.driverWaypoints.map(
      (waypoint: Point, index: number) =>
        new WayStep({
          lon: waypoint.lon,
          lat: waypoint.lat,
          actors: [
            new Actor({
              role: Role.DRIVER,
              target: this._getTarget(index, this.driverWaypoints),
            }),
          ],
        }),
    );

  private _createPassengerWaysteps = (): WayStep[] => {
    const waysteps: WayStep[] = [];
    this.passengerWaypoints.forEach(
      (passengerWaypoint: Point, index: number) => {
        const waystep: WayStep = new WayStep({
          lon: passengerWaypoint.lon,
          lat: passengerWaypoint.lat,
          actors: [
            new Actor({
              role: Role.PASSENGER,
              target: this._getTarget(index, this.passengerWaypoints),
            }),
          ],
        });
        if (
          this.driverWaypoints.filter((driverWaypoint: Point) =>
            passengerWaypoint.isSame(driverWaypoint),
          ).length > 0
        ) {
          waystep.actors.push(
            new Actor({
              role: Role.DRIVER,
              target: Target.NEUTRAL,
            }),
          );
        }
        waysteps.push(waystep);
      },
    );
    return waysteps;
  };

  private _createMixedWaysteps = (
    driverWaysteps: WayStep[],
    passengerWaysteps: WayStep[],
  ): WayStep[] =>
    driverWaysteps.length == 2
      ? [driverWaysteps[0], ...passengerWaysteps, driverWaysteps[1]]
      : this._createComplexMixedWaysteps(driverWaysteps, passengerWaysteps);

  private _createComplexMixedWaysteps = (
    driverWaysteps: WayStep[],
    passengerWaysteps: WayStep[],
  ): WayStep[] => [];

  private _getTarget = (index: number, waypoints: Point[]): Target =>
    index == 0
      ? Target.START
      : index == waypoints.length - 1
      ? Target.FINISH
      : Target.INTERMEDIATE;
}
